@@ -1,7 +1,8 @@
 """
 This file is used by asv_compare.conf.json.tpl.
 """
-import subprocess, sys
+import subprocess
+import sys
 # pip ignores '--global-option' when pep517 is enabled therefore we disable it.
 cmd = [sys.executable, '-mpip', 'wheel', '--no-use-pep517']
 try:
