@@ -1,16 +1,6 @@
+# SPDX-License-Identifier: Apache-2.0
 # Copyright 2013-2020 The Meson development team
 
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-
-#     http://www.apache.org/licenses/LICENSE-2.0
-
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
 from __future__ import annotations
 
 import re
@@ -28,7 +18,8 @@ from .pkgconfig import PkgConfigDependency
 from .misc import threads_factory
 
 if T.TYPE_CHECKING:
-    from ..environment import Environment, Properties
+    from ..envconfig import Properties
+    from ..environment import Environment
 
 # On windows 3 directory layouts are supported:
 # * The default layout (versioned) installed:
@@ -247,7 +238,7 @@ class BoostLibraryFile():
         # Handle the boost_python naming madness.
         # See https://github.com/mesonbuild/meson/issues/4788 for some distro
         # specific naming variations.
-        other_tags = []  # type: T.List[str]
+        other_tags: T.List[str] = []
 
         # Split the current modname into the base name and the version
         m_cur = BoostLibraryFile.reg_python_mod_split.match(self.mod_name)
@@ -330,9 +321,9 @@ class BoostLibraryFile():
         return True
 
     def get_compiler_args(self) -> T.List[str]:
-        args = []  # type: T.List[str]
+        args: T.List[str] = []
         if self.mod_name in boost_libraries:
-            libdef = boost_libraries[self.mod_name]  # type: BoostLibrary
+            libdef = boost_libraries[self.mod_name]
             if self.static:
                 args += libdef.static
             else:
@@ -354,19 +345,19 @@ class BoostDependency(SystemDependency):
         self.debug = buildtype.startswith('debug')
         self.multithreading = kwargs.get('threading', 'multi') == 'multi'
 
-        self.boost_root = None  # type: T.Optional[Path]
+        self.boost_root: T.Optional[Path] = None
         self.explicit_static = 'static' in kwargs
 
         # Extract and validate modules
-        self.modules = mesonlib.extract_as_list(kwargs, 'modules')  # type: T.List[str]
+        self.modules: T.List[str] = mesonlib.extract_as_list(kwargs, 'modules')
         for i in self.modules:
             if not isinstance(i, str):
                 raise DependencyException('Boost module argument is not a string.')
             if i.startswith('boost_'):
                 raise DependencyException('Boost modules must be passed without the boost_ prefix')
 
-        self.modules_found = []    # type: T.List[str]
-        self.modules_missing = []  # type: T.List[str]
+        self.modules_found: T.List[str] = []
+        self.modules_missing: T.List[str] = []
 
         # Do we need threads?
         if 'thread' in self.modules:
@@ -449,7 +440,7 @@ class BoostDependency(SystemDependency):
         mlog.debug('  - potential include dirs: {}'.format([x.path.as_posix() for x in inc_dirs]))
 
         #   2. Find all boost libraries
-        libs = []  # type: T.List[BoostLibraryFile]
+        libs: T.List[BoostLibraryFile] = []
         for i in lib_dirs:
             libs = self.detect_libraries(i)
             if libs:
@@ -470,8 +461,8 @@ class BoostDependency(SystemDependency):
                 mlog.debug(f'    - {j}')
 
             #   3. Select the libraries matching the requested modules
-            not_found = []  # type: T.List[str]
-            selected_modules = []  # type: T.List[BoostLibraryFile]
+            not_found: T.List[str] = []
+            selected_modules: T.List[BoostLibraryFile] = []
             for mod in modules:
                 found = False
                 for l in f_libs:
@@ -484,8 +475,8 @@ class BoostDependency(SystemDependency):
 
             # log the result
             mlog.debug('  - found:')
-            comp_args = []  # type: T.List[str]
-            link_args = []  # type: T.List[str]
+            comp_args: T.List[str] = []
+            link_args: T.List[str] = []
             for j in selected_modules:
                 c_args = j.get_compiler_args()
                 l_args = j.get_link_args()
@@ -523,7 +514,7 @@ class BoostDependency(SystemDependency):
         return False
 
     def detect_inc_dirs(self, root: Path) -> T.List[BoostIncludeDir]:
-        candidates = []  # type: T.List[Path]
+        candidates: T.List[Path] = []
         inc_root = root / 'include'
 
         candidates += [root / 'boost']
@@ -554,8 +545,8 @@ class BoostDependency(SystemDependency):
 
         # No system include paths were found --> fall back to manually looking
         # for library dirs in root
-        dirs = []     # type: T.List[Path]
-        subdirs = []  # type: T.List[Path]
+        dirs: T.List[Path] = []
+        subdirs: T.List[Path] = []
         for i in root.iterdir():
             if i.is_dir() and i.name.startswith('lib'):
                 dirs += [i]
@@ -577,7 +568,7 @@ class BoostDependency(SystemDependency):
         raw_list = dirs + subdirs
         no_arch = [x for x in raw_list if not any(y in x.name for y in arch_list_32 + arch_list_64)]
 
-        matching_arch = []  # type: T.List[Path]
+        matching_arch: T.List[Path] = []
         if '32' in self.arch:
             matching_arch = [x for x in raw_list if any(y in x.name for y in arch_list_32)]
         elif '64' in self.arch:
@@ -589,8 +580,8 @@ class BoostDependency(SystemDependency):
         # MSVC is very picky with the library tags
         vscrt = ''
         try:
-            crt_val = self.env.coredata.options[mesonlib.OptionKey('b_vscrt')].value
-            buildtype = self.env.coredata.options[mesonlib.OptionKey('buildtype')].value
+            crt_val = self.env.coredata.optstore.get_value('b_vscrt')
+            buildtype = self.env.coredata.optstore.get_value('buildtype')
             vscrt = self.clib_compiler.get_crt_compile_args(crt_val, buildtype)[0]
         except (KeyError, IndexError, AttributeError):
             pass
@@ -603,7 +594,8 @@ class BoostDependency(SystemDependency):
         # mlog.debug('    - vscrt: {}'.format(vscrt))
         libs = [x for x in libs if x.static == self.static or not self.explicit_static]
         libs = [x for x in libs if x.mt == self.multithreading]
-        libs = [x for x in libs if x.version_matches(lib_vers)]
+        if not self.env.machines[self.for_machine].is_openbsd():
+            libs = [x for x in libs if x.version_matches(lib_vers)]
         libs = [x for x in libs if x.arch_matches(self.arch)]
         libs = [x for x in libs if x.vscrt_matches(vscrt)]
         libs = [x for x in libs if x.nvsuffix != 'dll']  # Only link to import libraries
@@ -623,7 +615,7 @@ class BoostDependency(SystemDependency):
         return libs
 
     def detect_libraries(self, libdir: Path) -> T.List[BoostLibraryFile]:
-        libs = set()  # type: T.Set[BoostLibraryFile]
+        libs: T.Set[BoostLibraryFile] = set()
         for i in libdir.iterdir():
             if not i.is_file():
                 continue
@@ -654,16 +646,26 @@ class BoostDependency(SystemDependency):
         self.is_found = self.run_check([boost_inc_dir], [lib_dir])
 
     def detect_roots(self) -> None:
-        roots = []  # type: T.List[Path]
+        roots: T.List[Path] = []
 
         # Try getting the BOOST_ROOT from a boost.pc if it exists. This primarily
         # allows BoostDependency to find boost from Conan. See #5438
         try:
             boost_pc = PkgConfigDependency('boost', self.env, {'required': False})
             if boost_pc.found():
-                boost_root = boost_pc.get_pkgconfig_variable('prefix', [], None)
-                if boost_root:
-                    roots += [Path(boost_root)]
+                boost_lib_dir = boost_pc.get_variable(pkgconfig='libdir')
+                boost_inc_dir = boost_pc.get_variable(pkgconfig='includedir')
+                if boost_lib_dir and boost_inc_dir:
+                    mlog.debug('Trying to find boost with:')
+                    mlog.debug(f'  - boost_includedir = {Path(boost_inc_dir)}')
+                    mlog.debug(f'  - boost_librarydir = {Path(boost_lib_dir)}')
+
+                    self.detect_split_root(Path(boost_inc_dir), Path(boost_lib_dir))
+                    return
+                else:
+                    boost_root = boost_pc.get_variable(pkgconfig='prefix')
+                    if boost_root:
+                        roots += [Path(boost_root)]
         except DependencyException:
             pass
 
@@ -673,8 +675,9 @@ class BoostDependency(SystemDependency):
         inc_paths = [x.resolve() for x in inc_paths]
         roots += inc_paths
 
+        m = self.env.machines[self.for_machine]
         # Add system paths
-        if self.env.machines[self.for_machine].is_windows():
+        if m.is_windows():
             # Where boost built from source actually installs it
             c_root = Path('C:/Boost')
             if c_root.is_dir():
@@ -685,7 +688,7 @@ class BoostDependency(SystemDependency):
             # Where boost prebuilt binaries are
             local_boost = Path('C:/local')
 
-            candidates = []  # type: T.List[Path]
+            candidates: T.List[Path] = []
             if prog_files.is_dir():
                 candidates += [*prog_files.iterdir()]
             if local_boost.is_dir():
@@ -693,11 +696,15 @@ class BoostDependency(SystemDependency):
 
             roots += [x for x in candidates if x.name.lower().startswith('boost') and x.is_dir()]
         else:
-            tmp = []  # type: T.List[Path]
+            tmp: T.List[Path] = []
 
             # Add some default system paths
+            if m.is_darwin():
+                tmp.extend([
+                    Path('/opt/homebrew/'),        # for Apple Silicon MacOS
+                    Path('/usr/local/opt/boost'),  # for Intel Silicon MacOS
+                ])
             tmp += [Path('/opt/local')]
-            tmp += [Path('/usr/local/opt/boost')]
             tmp += [Path('/usr/local')]
             tmp += [Path('/usr')]
 
